const debug = require("debug")("titan:utils:network");
const request = require("request");
const os = require("os");

let IPv4,
  hostName;

hostName = os.hostname();
const interfaces = os.networkInterfaces();
for (const k in interfaces) {
  if (IPv4) break;
  for (const k2 in interfaces[k]) {
    const detail = interfaces[k][k2];
    if (detail.family === "IPv4" && !detail.internal && !/^127\.0/.test(detail.address)) {
      // console.log(detail);
      IPv4 = detail.address;
      break;
    }
  }
}
// 本地IP
exports.localip = IPv4;
// 主机名
exports.hostname = hostName;
exports.host = hostName;

// rpc 调用
exports._jsonrpc = async function (opt) {
  // 支持自动重试
  return new Promise(((resolve, reject) => {
    const { request } = exports;
    request(opt)
      .then((res) => {
        if (res.code == 0) {
          resolve(res.data || res.result);
        } else {
          reject(res);
        }
      })
      .catch((err) => {
        reject({
          code: 500,
          msg: err,
        });
      });
  }));
};

exports.jsonrpc = async function (opt) {
  opt.json = true;
  let result;
  try {
    result = await exports.request(opt);
  } catch (e) {
    result = {
      code: 500,
      msg: e,
    };
  }
  if (result != undefined && result.code == 0) {
    return result.data || result.result || result || {};
  } else {
    if (result == undefined) {
      return {code: -1};
    } else {
      return result;
    }
    //throw result;
  }
};

// 网络请求
exports.request = function (opt) {
  opt.proxy = null;
  return new Promise(((resolve, reject) => {
    request(opt, (err, xhr, body) => {
      if (err) return reject(err);
      resolve(body);
    });
  }));
};

let proxy;
exports.proxy = function (ctx, config) {
  // config { target: "xxxx" };
  if (!proxy) {
    const httpProxy = require("http-proxy");
    proxy = httpProxy.createProxyServer();
  }
  return new Promise((resolve, reject) => {
    proxy.web(ctx.req, ctx.res, config, (res) => {
      resolve(res);
    });
    proxy.on("error", (err) => {
      reject(err);
    });
  });
};
